"""Django email backend for celery."""

VERSION = (1, 1, 2)

__version__ = '.'.join(map(str, VERSION))
__author__ = 'Paul McLanahan'
__contact__ = 'paul@mclanahan.net'
__homepage__ = 'https://github.com/pmclanahan/django-celery-email'
__license__ = 'BSD (3 clause)'
